using System;
using System.Web;
using System.Web.UI;
using System.Threading;


public class CompositeAsyncResult : IAsyncResult
{
    #region Private members
    AsyncCallback _cb;
	object _state;
    AsyncCallback _completionCallBack;
	int _pendingOperations;

    // The volatile modifier is usually used for a field that is 
    // accessed by multiple threads without using the lock 
    // statement to serialize access 
    volatile bool _allCompletedSynchronously;
    #endregion
       
    
    // **********************************************************************************
    // Ctor
    public CompositeAsyncResult(AsyncCallback callback, object state, int operationCount)
    {
        _cb = callback;
        _state = state;
		_pendingOperations = operationCount;
        _allCompletedSynchronously = true;
        _completionCallBack = new AsyncCallback(this.CompletionCallback);
    }

    // **********************************************************************************
    // Returns the completion callback delegate
    public AsyncCallback Callback
	{
		get { return _completionCallBack; }
	}
        
    // **********************************************************************************
    // The built-in callback function invoked when a task terminates 
	public void CompletionCallback(IAsyncResult ar)
	{
		// If the current call is not terminated synchronously, set state properly 
		if (!ar.CompletedSynchronously)
		{
			_allCompletedSynchronously = false;
		}

        // Decrement and check if all tasks are complete
		if (Interlocked.Decrement(ref _pendingOperations) == 0)
		{
            // Done; call the ASP.NET callback we were passed through the ctor 
			_cb(this);
		}
    }

    #region IAsyncResult
    // **********************************************************************************
    // Returns the state object passed through the ctor
	object IAsyncResult.AsyncState 
    { 
        get { return _state; } 
    }
	
    // **********************************************************************************
    // Returns the wait-handle object we're using 
    WaitHandle IAsyncResult.AsyncWaitHandle	
    { 
        get { return null; } 
    }

    // **********************************************************************************    
	// Indicates whether all tasks terminated and terminated synchronously 
	bool IAsyncResult.CompletedSynchronously 
    {
        get { return (_pendingOperations == 0) && _allCompletedSynchronously; } 
    }

    // **********************************************************************************
    // Indicates whether the asynchronous operation has completed 
    bool IAsyncResult.IsCompleted 
    {
        get { return (_pendingOperations == 0); } 
    }
    #endregion
}


