using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Profile;
using System.Collections.Specialized;
using System.Web.Hosting;
using System.Configuration.Provider;
using System.Text;
using System.Collections;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;


namespace ProAspNet20.Advanced.CS.Components
{
    public class CookieProfileProvider : ProfileProvider
    {
        #region Private Members
        private string _cookieName;
        private int _cookieExpires;
        private string _appName;
        #endregion

        public CookieProfileProvider()
        {
        }


        #region Properties
        public override string ApplicationName
        {
            get { return _appName; }
            set { _appName = value; }
        }

        public virtual string CookieName
        {
            get { return _cookieName; }
            set { _cookieName = value; }
        }

        public virtual int CookieExpires
        {
            get { return _cookieExpires; }
            set { _cookieExpires = value; }
        }
        #endregion


        #region Initialize()
        public override void Initialize(string name, NameValueCollection config)
        {
            // Check the nullness of configuration attributes
            if (config == null)
                throw new ArgumentNullException("Null configuration parameters");

            // Make sure the provider has a name 
            if (String.IsNullOrEmpty(name))
                name = "CookieProfileProvider";

            // Call the base class method
            base.Initialize(name, config);


            // Consumed attribute MUST be removed from the collection


            // Read the application name
            _appName = config["applicationName"];
            if (String.IsNullOrEmpty(_appName))
                _appName = HostingEnvironment.ApplicationVirtualPath;
            config.Remove("applicationName");

            // Read the cookie name
            _cookieName = config["cookieName"];
            if (String.IsNullOrEmpty(_cookieName))
                _cookieName = ".PROF";
            config.Remove("cookieName");

            // Read the cookie expiration
            bool success = Int32.TryParse(config["cookieExpires"], out _cookieExpires);
            if (!success)
                _cookieExpires = 10;
            config.Remove("cookieExpires");

            // Throw an exception if there are still unknown attributes
            if (config.Count > 0)
            {
                string attrib = config.GetKey(0);
                if (!String.IsNullOrEmpty(attrib))
                    throw new ProviderException("Unrecognized attribute: " + attrib);
            }
        }
        #endregion


        #region GetPropertyValues()
        public override SettingsPropertyValueCollection GetPropertyValues(SettingsContext context, SettingsPropertyCollection properties)
        {
            SettingsPropertyValueCollection settings = new SettingsPropertyValueCollection();

            // Check how many properties are we going to retrieve
            if (properties.Count == 0)
                return settings;

            // Add all properties to the output collection. This guarantees that as many entries will
            // be created (and not be null) on the Profile object
            foreach (SettingsProperty property in properties)
            {
                SettingsPropertyValue pv = new SettingsPropertyValue(property);
                settings.Add(pv);
            }

            // Read the cookie (if any)
            string username = (string)context["UserName"];
            string cookie = CookieName + "_" + username;
            HttpCookie cookieProfile = HttpContext.Current.Request.Cookies[cookie];
            if (cookieProfile == null)
                return settings;

            // Base64 decoding and binary deserialization
            string data = cookieProfile["SerializedData"];
            byte[] bits = Convert.FromBase64String(data);
            MemoryStream mem = new MemoryStream(bits);
            BinaryFormatter bin = new BinaryFormatter();
            Hashtable table = (Hashtable)bin.Deserialize(mem);
            mem.Close();

            // Add data to the properties
            foreach (SettingsPropertyValue spv in settings)
            {
                // Value is deserialized
                spv.PropertyValue = table[spv.Name];
                spv.Deserialized = true;
            }
          
            return settings;
        }
        #endregion


        #region SetPropertyValues()
        public override void SetPropertyValues(SettingsContext context, SettingsPropertyValueCollection properties)
        {
            // Get information about the user 
            string username = (string)context["UserName"];
            bool authenticated = (bool)context["IsAuthenticated"];

            // If no properties, return 
            if (String.IsNullOrEmpty(username) || properties.Count == 0)
                return;

            // Prepare the cookie
            string cookie = CookieName + "_" + username;
            HttpCookie cookieProfile = HttpContext.Current.Request.Cookies[cookie];
            if (cookieProfile == null)
                cookieProfile = new HttpCookie(cookie);
            cookieProfile.Expires = DateTime.Now.AddMinutes(CookieExpires);

            // Prepare the data to store
            Hashtable table = new Hashtable();
            foreach (SettingsPropertyValue pp in properties)
            {
                if (!authenticated && !(bool)pp.Property.Attributes["AllowAnonymous"])
                    continue;
                table.Add(pp.Name, pp.PropertyValue);
            }

            // Encode and write the hashtable to the cookie
            BinaryFormatter bin = new BinaryFormatter();
            MemoryStream mem = new MemoryStream();
            bin.Serialize(mem, table);
            string data = Convert.ToBase64String(mem.GetBuffer(), 0, (int)mem.Length);
            cookieProfile["SerializedData"] = data;

            // Save
            HttpContext.Current.Response.AppendCookie(cookieProfile);
        }
        #endregion



        #region Not Implemented Methods
        public override int DeleteInactiveProfiles(ProfileAuthenticationOption authenticationOption, DateTime userInactiveSinceDate)
        {
            throw new NotSupportedException();
        }

        public override int DeleteProfiles(string[] usernames)
        {
            throw new NotSupportedException();
        }

        public override int DeleteProfiles(ProfileInfoCollection profiles)
        {
            throw new NotSupportedException();
        }

        public override ProfileInfoCollection FindInactiveProfilesByUserName(ProfileAuthenticationOption authenticationOption, string usernameToMatch, DateTime userInactiveSinceDate, int pageIndex, int pageSize, out int totalRecords)
        {
            throw new NotSupportedException();
        }

        public override ProfileInfoCollection FindProfilesByUserName(ProfileAuthenticationOption authenticationOption, string usernameToMatch, int pageIndex, int pageSize, out int totalRecords)
        {
            throw new NotSupportedException();
        }

        public override ProfileInfoCollection GetAllInactiveProfiles(ProfileAuthenticationOption authenticationOption, DateTime userInactiveSinceDate, int pageIndex, int pageSize, out int totalRecords)
        {
            throw new NotSupportedException();
        }

        public override ProfileInfoCollection GetAllProfiles(ProfileAuthenticationOption authenticationOption, int pageIndex, int pageSize, out int totalRecords)
        {
            throw new NotSupportedException();
        }

        public override int GetNumberOfInactiveProfiles(ProfileAuthenticationOption authenticationOption, DateTime userInactiveSinceDate)
        {
            throw new NotSupportedException();
        }
        #endregion

    }
}