using System;
using System.ComponentModel;
using System.ComponentModel.Design.Serialization;
using System.Drawing;
using System.Globalization;
using System.Drawing.Design;
using System.Windows.Forms;
using System.Web.UI.Design;
using System.Web.UI.Design.WebControls;
using System.Web.UI.WebControls;


namespace ProAspNet20.Advanced.CS.Components.Design
{
    #region CLASS::GaugeFilterSettingsConverter
    public class GaugeFilterSettingsConverter : ExpandableObjectConverter
    {
        private const int NumOfMembers = 4;

        public GaugeFilterSettingsConverter()
        {
        }

        // **************************************************************************
        // Conversion from String is possible
        public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
        {
            if (sourceType == typeof(string))
                return true;

            return base.CanConvertFrom(context, sourceType);
        }

        // **************************************************************************
        // Conversion to String is possible and new instances can be created
        public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
        {
            if (destinationType == typeof(string))
                return true;

            if (destinationType == typeof(InstanceDescriptor))
                return true;

            return base.CanConvertTo(context, destinationType);
        }

        // **************************************************************************
        // Convert a FilterSettings to a string 
        public override object ConvertTo(ITypeDescriptorContext context,
               CultureInfo culture, object value, Type destinationType)
        {
            // Ensure the value is a FilterSettings object
            if (value != null)
            {
                if (!(value is FilterSettings))
                    throw new ArgumentException(
                            "Invalid FilterSettings object", "value");
            }

            // If null, return the empty string
            if (destinationType == typeof(string))
            {
                if (value == null)
                    return String.Empty;

                // Get the object to serialize and necessary converters
                FilterSettings filter = (FilterSettings)value;
                TypeConverter cvInt;
                cvInt = TypeDescriptor.GetConverter(typeof(int));
                TypeConverter cvColor;
                cvColor = TypeDescriptor.GetConverter(typeof(Color));

                // Create the string
                string[] parts = new string[NumOfMembers];
                parts[0] = filter.FilterType.ToString();
                parts[1] = (string) cvInt.ConvertTo(filter.AlphaOpacity, typeof(int));
                parts[2] = (string) cvInt.ConvertTo(filter.GlowStrength, typeof(int));
                parts[3] = (string) cvColor.ConvertTo(filter.GlowColor, typeof(string));
                return String.Join(culture.TextInfo.ListSeparator, parts);
            }

            return base.ConvertTo(context, culture, value, destinationType);
        }

        // **************************************************************************
        // Convert the specified value to a FilterSettings object
        public override object ConvertFrom(ITypeDescriptorContext context,
               CultureInfo culture, object value)
        {
            // Default instance
            if (value == null)
                return new FilterSettings();

            // Deserialize from a string representation
            if (value is string)
            {
                // Get the source string
                string data = (string)value;

                if (data.Length == 0)
                    return new FilterSettings();

                // Split the string into parts  
                string[] parts = data.Split(culture.TextInfo.ListSeparator[0]);
                if (parts.Length != NumOfMembers)
                    throw new ArgumentException(
                            "Invalid FilterSettings object", "value");

                // Create a new FilterSettings object
                FilterSettings filter = new FilterSettings();

                // Part 0 is a string representing the FilterType property
                Type t = typeof(GaugeFilterType);
                filter.FilterType = (GaugeFilterType)Enum.Parse(t, parts[0]);

                // Part 1 represents the AlphaOpacity property (integer) 
                TypeConverter cvInt;
                cvInt = TypeDescriptor.GetConverter(typeof(int));
                filter.AlphaOpacity = (int) cvInt.ConvertFrom(context, culture,
                             parts[1]);

                // Part 2 represents the GlowStrength property (integer) 
                filter.GlowStrength = (int) cvInt.ConvertFrom(context, culture,
                             parts[2]);

                // Part 3 represents the GlowColor property (Color)
                TypeConverter cvColor;
                cvColor = TypeDescriptor.GetConverter(typeof(Color));
                filter.GlowColor = (Color) cvColor.ConvertFrom(context, culture, parts[3]);

                return filter;
            }

            return base.ConvertFrom(context, culture, value);
        }
    }
    #endregion



    #region CLASS::GaugeFilterSettingsEditor
    public class GaugeFilterSettingsEditor : UITypeEditor
    {
        public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context)
        {
            return UITypeEditorEditStyle.Modal;
        }


        public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value)
        {
            MessageBox.Show("Hello");
            //FilterSettingsDialog dlg = new FilterSettingsDialog();
            //dlg.ShowDialog();

            return value;
        }
    }
    #endregion



    #region CLASS::GaugeBarDesigner
    public class GaugeBarDesigner : System.Web.UI.Design.ControlDesigner
    {
        #region Private Members
        private GaugeBar _controlInstance;
        private bool _addNote;
        private float _originalValue;
        #endregion

        protected override string GetErrorDesignTimeHtml(Exception e)
        {
            return e.StackTrace;
        }

        public override void Initialize(IComponent component)
        {
            // get a reference to the control to design
            _controlInstance = (GaugeBar) component;
            base.Initialize(component);
        }


        public override string GetDesignTimeHtml()
        {
            if (_controlInstance.Value < 0)
            {
                // Save the current (invalid) Value
                _originalValue = _controlInstance.Value;

                // generate a random value
                int v = GetRandomNumber();

                // This assignment must be undone on exit 
                // otherwise it'll be saved to the ASPX
                _controlInstance.Value = v;

                _addNote = true;
            }
            else
                _addNote = false;


            // If no valid Value is provided, add a note to the markup
            if (_addNote)
            {
                string template = "<table cellpadding=4 cellspacing=0 style=\"font:messagebox;color:buttontext;background-color:buttonface;border: solid 1px;border-top-color:buttonhighlight;border-left-color:buttonhighlight;border-bottom-color:buttonshadow;border-right-color:buttonshadow\"><tr><td nowrap><span style=\"font-weight:bold\">{0}</span> - {1}</td></tr><tr><td>{2}</td></tr><tr><td>{3}</td></tr</table>";

                string message = "Die <b>Value</b> Eigenschaft wurde auf keinen gltigen Wert gesetzt. <br/>Der Anzeigewert wurde nach dem Zufallsprinzip generiert, um Ihnen eine Vorstellung vom Steuerelement zu vermitteln. <hr/><small>Mit freundlicher Genehmigung von <b>Programming Microsoft ASP.NET 2.0 ApplicationsAdvanced Topics</b></small>";
                string markup =  String.Format(template,
                    base.Component.GetType().Name,
                    base.Component.Site.Name,
                    base.GetDesignTimeHtml(),
                    message);

                // Restore the original value
                _controlInstance.Value = _originalValue;

                // Return
                return markup;
            }

            // Return the default markup
            return base.GetDesignTimeHtml();
        }


        protected override void OnClick(DesignerRegionMouseEventArgs e) 
        {
            base.OnClick(e);

            // Re-render the control to show the highlight
            UpdateDesignTimeHtml();
        }

        
        public override DesignerAutoFormatCollection AutoFormats
        {
            get
            {
                DesignerAutoFormatCollection autoFormats = new DesignerAutoFormatCollection();
                autoFormats.Add(new SimpleAutoFormat());
                autoFormats.Add(new PercentageAutoFormat());
                return autoFormats;
            }
        }

        #region Helpers
        private  int GetRandomNumber()
        {
            Random rnd = new Random();
            return (int) rnd.Next(0, (int)_controlInstance.Maximum);
        }
        #endregion
    }

    #region AutoFormat Classes
    public class SimpleAutoFormat : DesignerAutoFormat
    {
        public SimpleAutoFormat() : base("(Standardformatierung)")
        {
        }

        public override void Apply(System.Web.UI.Control control)
        {
            GaugeBar _gaugeBar = control as GaugeBar;
            if (_gaugeBar != null)
            {
                _gaugeBar.FormatString = "{0} out of {1}";
                _gaugeBar.Segments = 4;
                _gaugeBar.TextStyle.BackColor = Color.Transparent;
                _gaugeBar.TextStyle.Font.Name = "verdana";
                _gaugeBar.TextStyle.Font.Size = FontUnit.Point(8);
                _gaugeBar.TextStyle.DisplayTextAtBottom = true;
                _gaugeBar.TextStyle.RenderInsideTable = false;
            }
        }
    }

    public class PercentageAutoFormat : DesignerAutoFormat
    {
        public PercentageAutoFormat() : base("(Prozentsatz-Formatierung)")
        {
        }

        public override void Apply(System.Web.UI.Control control)
        {
            GaugeBar _gaugeBar = control as GaugeBar;
            if (_gaugeBar != null)
            {
                _gaugeBar.FormatString = "{0} %";
                _gaugeBar.Segments = 10;
                _gaugeBar.TextStyle.BackColor = Color.Transparent;
                _gaugeBar.TextStyle.Font.Name = "verdana";
                _gaugeBar.TextStyle.Font.Size = FontUnit.Point(8);
                _gaugeBar.TextStyle.DisplayTextAtBottom = false;
            }
        }
    }
    #endregion
    
    #endregion
}