#region Using directives
using System;
using System.Data;
using System.Data.SqlClient;
using System.Collections.Generic;
using System.CodeDom;
using System.Web.Compilation;
using System.Xml;
using System.IO;
using System.Web.Hosting;
#endregion


namespace ProAspNet20.Advanced.CS.Components
{
	public class OrmBuildProvider : BuildProvider
	{
		public OrmBuildProvider()
		{
		}


		#region BuildProvider overrides
		public override void GenerateCode(AssemblyBuilder ab)
		{
			// Get the path of the source file to be built
			string fileName = base.VirtualPath;

			// Can use a text-writer object to write code directly
			//TextWriter tw = ab.CreateCodeFile(this);
			//tw.Write(sourceClass); 
			//tw.Close();

			// Get the tree representing the generated code
			CodeCompileUnit code = BuildCodeTree(fileName);

			// Build an assembly using the code tree
			ab.AddCodeCompileUnit(this, code);

			// AssemblyBuilder knows about the compiler to use
		}
		#endregion


		#region Helper methods
		// **************************************************************************
		// METHOD: BuildCodeTree
		// Manage the process that creates the code tree from the .map file
		private CodeCompileUnit BuildCodeTree(string fileName)
		{
			OrmDescriptor desc = ExtractInfo(fileName);
			return GenerateCodeDomTree(desc);
		}


		// **************************************************************************
		// METHOD: ExtractInfo
		// Read the .map file and copies information into a tailormade structure
		private OrmDescriptor ExtractInfo(string fileName)
		{
			// Load the document
			XmlDocument doc = new XmlDocument();
			using (Stream file = VirtualPathProvider.OpenFile(fileName))
			{
				doc.Load(file);
			}

			// Get namespace information
			XmlNode root = doc.DocumentElement;
			string ns = root.Attributes["namespace"].Value;
 
			// Visit the <mapping> nodes
			XmlNodeList mappings = doc.SelectNodes("mappings/mapping");
			OrmTableDescriptor[] descriptors = new OrmTableDescriptor[mappings.Count];
			for(int i=0; i<descriptors.Length; i++)
			{
				XmlNode mapping = mappings[i];
				OrmTableDescriptor tblDescriptor = new OrmTableDescriptor();
				descriptors[i] = tblDescriptor;

				tblDescriptor.ConnectionString = mapping.Attributes["connectionString"].Value;
				tblDescriptor.ClassName = mapping.Attributes["className"].Value;
				tblDescriptor.TableName = mapping.Attributes["tableName"].Value;
				tblDescriptor.SelectCommand = mapping.Attributes["selectCommand"].Value;
				
				bool isPartial = false;
				Boolean.TryParse(mapping.Attributes["allowPartialClass"].Value, out isPartial);
				tblDescriptor.IsPartial = isPartial;

				bool allowCollection = false;
                Boolean.TryParse(mapping.Attributes["allowCollectionClass"].Value, out allowCollection);
				tblDescriptor.AllowCollectionClass = allowCollection;
				if (allowCollection)
					tblDescriptor.CollectionClassName = mapping.Attributes["collectionClassName"].Value;

				bool allowGateway = false;
				tblDescriptor.AllowGatewayClass = false;
				if (Boolean.TryParse(mapping.Attributes["allowGatewayClass"].Value, out allowGateway))
					tblDescriptor.AllowGatewayClass = allowGateway;
				if (allowGateway)
					tblDescriptor.GatewayClassName = mapping.Attributes["gatewayClassName"].Value;
			}

			// Pack all info and return
			OrmDescriptor desc = new OrmDescriptor();
			desc.Namespace = ns;
			desc.Descriptors = descriptors;
			
			return desc;
		}


		// **************************************************************************
		// METHOD: GenerateCodeDomTree
		// Uses the provided information to create a CodeDOM tree
		private CodeCompileUnit GenerateCodeDomTree(OrmDescriptor desc)
		{
			CodeCompileUnit code = new CodeCompileUnit();

			// Import namespaces
			CodeNamespaceImport unitImport = new CodeNamespaceImport("System.Collections.Generic");
			
			// Create the namespace
			CodeNamespace unitNamespace = new CodeNamespace(desc.Namespace);
			unitNamespace.Imports.Add(unitImport);
			code.Namespaces.Add(unitNamespace);

			// Loop through the mappings and add classes
			for (int i = 0; i < desc.Descriptors.Length; i++)
			{
				// Get the descriptor
				OrmTableDescriptor tblDescriptor = desc.Descriptors[i];

				// Create the class to represent the table
		        CodeTypeDeclaration unitClass = new CodeTypeDeclaration(tblDescriptor.ClassName);
				unitClass.IsPartial = tblDescriptor.IsPartial;
				unitNamespace.Types.Add(unitClass);

				// Loop through the selected table columns and add members
				SqlDataAdapter adapter = new SqlDataAdapter(tblDescriptor.SelectCommand, 
					tblDescriptor.ConnectionString);
				DataTable dt = new DataTable();
				adapter.FillSchema(dt, SchemaType.Mapped);

				for(int j=0; j<dt.Columns.Count; j++)
				{
					DataColumn column = dt.Columns[j];
					string colName = column.ColumnName;
					Type colType = column.DataType;
					string fieldName = "_" + colName.ToLower();

					//
					// Add a property for each column
					//

					// Add the private field to store the data
					CodeMemberField unitField = new CodeMemberField(colType, fieldName);
					unitClass.Members.Add(unitField);

					// Add property declaration and get/set accessors
					CodeMemberProperty unitProperty = new CodeMemberProperty();
					unitProperty.Name = column.ColumnName;
					unitProperty.Type = new CodeTypeReference(column.DataType);
					unitProperty.Attributes = MemberAttributes.Public;
	
					// Define the codeDOM reference for the property's private field 
					CodeFieldReferenceExpression unitFieldRef = new CodeFieldReferenceExpression();
					unitFieldRef.TargetObject = new CodeThisReferenceExpression();
					unitFieldRef.FieldName = fieldName;

					// Get
					CodeMethodReturnStatement unitReturn = new CodeMethodReturnStatement(unitFieldRef);
					unitProperty.GetStatements.Add(unitReturn);
				
					// Set
					CodeAssignStatement unitAssign = new CodeAssignStatement();
					unitAssign.Left = unitFieldRef;
					unitAssign.Right = new CodePropertySetValueReferenceExpression();
					unitProperty.SetStatements.Add(unitAssign);

					unitClass.Members.Add(unitProperty);
				}

				// Create the collection class if required
				if (tblDescriptor.AllowCollectionClass)
				{
					string collClassName = tblDescriptor.CollectionClassName;
					CodeTypeDeclaration unitCollClass = new CodeTypeDeclaration(collClassName);
					CodeTypeReference type = new CodeTypeReference("List", CodeTypeReferenceOptions.GenericTypeParameter);
					type.TypeArguments.Add(tblDescriptor.ClassName);
					unitCollClass.BaseTypes.Add(type);
					unitNamespace.Types.Add(unitCollClass);
				}

				// Create the gateway class if required
				if (tblDescriptor.AllowGatewayClass)
				{
					string gatewayClassName = tblDescriptor.GatewayClassName;
					CodeTypeDeclaration unitGatewayClass = new CodeTypeDeclaration(gatewayClassName);
					unitGatewayClass.IsPartial = true;
					unitNamespace.Types.Add(unitGatewayClass);

					// Add a free ConnectionString property
					CodeMemberProperty unitConnStringProperty = new CodeMemberProperty();
					unitConnStringProperty.Name = "ConnectionString";
					unitConnStringProperty.Type = new CodeTypeReference(typeof(string));
					unitConnStringProperty.Attributes = MemberAttributes.Public | MemberAttributes.Static;

					// Get
					CodePrimitiveExpression unitExpr = new CodePrimitiveExpression(tblDescriptor.ConnectionString);
					CodeMethodReturnStatement unitConnStringReturn = new CodeMethodReturnStatement(unitExpr);
					unitConnStringProperty.GetStatements.Add(unitConnStringReturn);

					unitGatewayClass.Members.Add(unitConnStringProperty);
				}
			}

			return code;
		}
		#endregion
	}



	#region CLASS OrmDescriptor
	class OrmDescriptor
	{
		public string Namespace;
		public OrmTableDescriptor[] Descriptors;
	}
	#endregion


	#region CLASS OrmTableDescriptor
	class OrmTableDescriptor
	{
		public string ConnectionString;
		public string TableName;
		public string ClassName;
		public string SelectCommand;
		public bool IsPartial;
		public bool AllowCollectionClass;
		public string CollectionClassName;
		public bool AllowGatewayClass;
		public string GatewayClassName;
	}
	#endregion
}
