using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.Drawing;



namespace ProAspNet20.Advanced.CS.Components
{
    [DefaultProperty("Value")]
    [ToolboxData("<{0}:SimpleGaugeBar runat=\"server\" width=\"100px\" height=\"25px\" />")]
    public class SimpleGaugeBar : CompositeControl 
    {
        private TextItemStyle _textStyle;
        private int _dividerCell;


        // *******************************************************
        // Ctor
        public SimpleGaugeBar()
        {
        }

        #region Properties
        // *******************************************************
        // Gets and sets the value to represent in the gauge bar
        public float Value
        {
            get
            {
                object o = ViewState["Value"];
                if (o == null)
                    return 0;
                return (float) o;
            }
            set 
            {
                ViewState["Value"] = value;
                if (value > Maximum)
                    ViewState["Value"] = Maximum;
            }
        }

        // *******************************************************
        // Gets and sets the maximum value representable in the gauge bar
        public float Maximum
        {
            get
            {
                object o = ViewState["Maximum"];
                if (o == null)
                    return 100;
                return (float) o;
            }
            set { ViewState["Maximum"] = value; }
        }

        // *******************************************************
        // Number of segments to divide the bar into
        public int Segments
        {
            get
            {
                object o = ViewState["Segments"];
                if (o == null)
                    return 4;
                return (int) o;
            }
            set 
            { 
                ViewState["Segments"] = value;
                if( value < 1)
                    ViewState["Segments"] = 1; 
            }
        }

        // *******************************************************
        // Gets and sets the pattern to format the value in the gauge bar
        public string FormatString
        {
            get
            {
                object o = ViewState["FormatString"];
                if (o == null)
                    return "<b>{0}</b> / <b>{1}</b>";
                return (string) o;
            }
            set { ViewState["FormatString"] = value; }
        }

        // *******************************************************
        // Gets and sets whether the gaugebar has grid lines
        public bool GridLines
        {
            get
            {
                object o = ViewState["GridLines"];
                if (o == null)
                    return true;
                return (bool) o;
            }
            set { ViewState["GridLines"] = value; }
        }


        // ***************************************************************
        // Gets the style of the gauge label
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        [PersistenceMode(PersistenceMode.InnerProperty)]
        public TextItemStyle TextStyle
        {
            get
            {
                if (_textStyle == null)
                    _textStyle = new TextItemStyle();

                if (IsTrackingViewState)
                    ((IStateManager)_textStyle).TrackViewState();

                return _textStyle;
            }
        }
        #endregion


        #region ViewState Management
        // We should override LoadViewState and SaveViewState to
        // guarantee that style information (TextStyle) is correctly
        // persisted on the viewstate across postbacks.
        // We'll do this in the GaugeBar control--an enhanced version of this
        #endregion


        #region Rendering
        // *******************************************************
        // Renders the output of the control
        protected override void Render(HtmlTextWriter output)
        {
            PrepareControlForRendering();
            base.RenderContents(output);
        }

        // *******************************************************
        // Style the control 
        protected virtual void PrepareControlForRendering()
        {
            // Make sure there are controls to work with
            if (Controls.Count < 1)
                return;

            // Get the table
            Table outer = (Table)Controls[0];
            Table t = (Table)outer.Rows[0].Cells[0].Controls[0];

            // Apply the table style
            t.CopyBaseAttributes(this);
            if (ControlStyleCreated)
                t.ApplyStyle(ControlStyle);

            // Configure additional property on the overall table
            t.CellPadding = 0;
            t.CellSpacing = 0;

            // Set colors on the cells
            for (int i = 0; i < Segments; i++)
            {
                TableCell cell = t.Rows[0].Cells[i];
                if (GridLines)
                {
                    cell.BorderColor = BorderColor;
                    cell.BorderStyle = BorderStyle;
                    cell.BorderWidth = BorderWidth;
                }

                if (i < _dividerCell)
                    cell.BackColor = ForeColor;
                if (i >= _dividerCell)
                    cell.BackColor = BackColor;
                if (i == _dividerCell)
                {
                    Table inner = (Table)cell.Controls[0];
                    if (inner.Rows[0].Cells.Count > 1)
                    {
                        inner.Rows[0].Cells[0].BackColor = ForeColor;
                        inner.Rows[0].Cells[1].BackColor = BackColor;
                    }
                    else
                    {
                        // ratio in the divider cell is 0-100 
                        // (i.e., a value of 25 on 100 with 4 segments)
                        inner.Rows[0].Cells[0].BackColor = BackColor;
                    }
                }
            }

            // Style the gauge bar text style
            if (!_textStyle.DisplayTextAtBottom)
            {
                // extra cell
                outer.Rows[0].Cells[1].ApplyStyle(TextStyle);
            }
            else if (_textStyle.RenderInsideTable && _textStyle.DisplayTextAtBottom)
            {
                // bottom row
                TableRow row = t.Rows[1];
                row.ApplyStyle(TextStyle);
            }
            else
            {
                // text below
                Label lbl = (Label)Controls[1];
                lbl.ApplyStyle(TextStyle);
            }

          }

        // *******************************************************
        // Control the creation of child controls
        protected override void CreateChildControls()
        {
 	      //  base.CreateChildControls();
            Controls.Clear();
            CreateControlHierarchy();
            ClearChildViewState();
        }

        // *******************************************************
        // Control the creation of child controls
        protected virtual void CreateControlHierarchy()
        {
            // Build the outermost container table
            Table outer = new Table();
            TableRow outerRow = new TableRow();
            outer.Rows.Add(outerRow);

            // Ruler cell
            TableCell rulerCell = new TableCell();
            outerRow.Cells.Add(rulerCell);
            BuildGaugeBar(rulerCell);

            // Text cell
            TableCell textCell = new TableCell();
            if (!_textStyle.DisplayTextAtBottom)
            {
                outerRow.Cells.Add(textCell);
                BuildLabel(textCell);
            }

            // Save the control tree
            Controls.Add(outer);

            // Build the label
            if (!_textStyle.RenderInsideTable && _textStyle.DisplayTextAtBottom)
                BuildLabel(null);
        }

        // *******************************************************
        // Build the control tree for the whole gauge bar 
        void BuildGaugeBar(TableCell container)
        {
            // Create the table with one or two rows: ruler (and label)
            Table t = new Table();
            TableRow ruler = new TableRow();
            t.Rows.Add(ruler);

            // Build the ruler row
            BuildRuler(ruler);

            // Build the label
            if (_textStyle.RenderInsideTable)
                BuildLabelIntoTable(t);

            // Save the control tree
            container.Controls.Add(t);
        }

        // *******************************************************
        // Build the control tree for the ruler 
        void BuildRuler(TableRow ruler)
        {
            // Calculate the value to represent
            float buf = GetValueToRepresent();
            float valueToRepresent = 100f * buf / Maximum;
            int numOfSegments = GetNumOfSegments();

            int segmentWidth = 100 / numOfSegments;
            bool finished = false;

            for (int i = 1; i <= numOfSegments; i++)
            {
                if (valueToRepresent < i * segmentWidth)
                {
                    if (finished)
                    {
                        // Still-To-Do
                        TableCell stillToDo = new TableCell();
                        ruler.Cells.Add(stillToDo);
                        stillToDo.Width = Unit.Percentage(segmentWidth);
                    }
                    else
                    {
                        // Cell to divide
                        _dividerCell = i - 1;  // need a 0-based index
                        TableCell cell = new TableCell();
                        ruler.Cells.Add(cell);
                        cell.Width = Unit.Percentage(segmentWidth);
                        cell.Height = Unit.Percentage(100);

                        // Add a child table to the cell
                        Table child = new Table();
                        child.Width = Unit.Percentage(100);
                        child.Height = Unit.Percentage(100);
                        cell.Controls.Add(child);
                        child.CellPadding = 0;
                        child.CellSpacing = 0;
                        TableRow childRow = new TableRow();
                        child.Rows.Add(childRow);

                        float fx = (100 * (valueToRepresent - segmentWidth * (i - 1)) / segmentWidth);
                        if (valueToRepresent > (i - 1) * segmentWidth)
                        {
                            TableCell left = new TableCell();
                            childRow.Cells.Add(left);
                            left.Width = Unit.Percentage(fx);
                        }
                        TableCell right = new TableCell();
                        childRow.Cells.Add(right);
                        right.Width = Unit.Percentage(100 - fx);
                        finished = true;
                    }
                }
                else
                {
                    // Done
                    TableCell done = new TableCell();
                    ruler.Cells.Add(done);
                    done.Width = Unit.Percentage(segmentWidth);
                }
            }
        }

        // *******************************************************
        // Build the control tree for the label 
        void BuildLabel(TableCell container)
        {
            // Calculate the value to represent
            float buf = GetValueToRepresent();

            // Get the string to display on the label 
            string msg = GetTextToRepresent();

            Label lbl = new Label();
            if (container is TableCell)
                container.Controls.Add(lbl);
            else
                Controls.Add(lbl);
            lbl.Text = String.Format(msg, buf, Maximum);
        }

        // *******************************************************
        // Build the control tree for the label 
        void BuildLabelIntoTable(Table t)
        {
            // Calculate the value to represent
            float buf = GetValueToRepresent();
            int numOfSegments = GetNumOfSegments();

            // Get the string to display on the label 
            string msg = GetTextToRepresent();

            if (_textStyle.DisplayTextAtBottom)
            {
                // Add a bottom row
                TableRow label = new TableRow();
                t.Rows.Add(label);

                TableCell lblCell = new TableCell();
                label.Cells.Add(lblCell);

                lblCell.ColumnSpan = numOfSegments;
                lblCell.Text = String.Format(msg, buf, Maximum);
            }
        }

        #region Helpers

        // *******************************************************
        // Helper function
        float GetValueToRepresent()
        {
            return Value;
        }

        // *******************************************************
        // Helper function
        string GetTextToRepresent()
        {
            return FormatString;
        }

        // *******************************************************
        // Helper function
        int GetNumOfSegments()
        {
            return Segments;
        }
        
        #endregion

        #endregion
    }
}
